@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """container.py - classes necessary to contain Terminal widgets"""
@@ -6,11 +5,11 @@
 from gi.repository import GObject
 from gi.repository import Gtk
 
-from factory import Factory
-from config import Config
-from util import dbg, err
-from translation import _
-from signalman import Signalman
+from .factory import Factory
+from .config import Config
+from .util import dbg, err
+from .translation import _
+from .signalman import Signalman
 
 # pylint: disable-msg=R0921
 class Container(object):
@@ -68,6 +67,16 @@ class Container(object):
         child is .remove()d and .add()ed"""
         return None
 
+    def split_auto(self, widget, cwd=None):
+        """Split this container automatically"""
+        width  = widget.get_allocation().width
+        height = widget.get_allocation().height
+        dbg("split as per current dimenstions:(%s %s)" % (width, height))
+        if width > height:
+            self.split_axis(widget, False, cwd)
+        else:
+            self.split_axis(widget, True , cwd)
+
     def split_horiz(self, widget, cwd=None):
         """Split this container horizontally"""
         return(self.split_axis(widget, True, cwd))
@@ -115,7 +124,7 @@ class Container(object):
         try:
             if self.get_property('term_zoomed'):
                 # We're zoomed, so unzoom and then start closing again
-                dbg('Container::closeterm: terminal zoomed, unzooming')
+                dbg('terminal zoomed, unzooming')
                 self.unzoom(widget)
                 widget.close()
                 return(True)
@@ -123,7 +132,7 @@ class Container(object):
             pass
 
         if not self.remove(widget):
-            dbg('Container::closeterm: self.remove() failed for %s' % widget)
+            dbg('self.remove() failed for %s' % widget)
             return(False)
 
         self.terminator.deregister_terminal(widget)
@@ -135,17 +144,6 @@ class Container(object):
         """Handle a keyboard event requesting a terminal resize"""
         raise NotImplementedError('resizeterm')
 
-    def toggle_zoom(self, widget, fontscale = False):
-        """Toggle the existing zoom state"""
-        try:
-            if self.get_property('term_zoomed'):
-                self.unzoom(widget)
-            else:
-                self.zoom(widget, fontscale)
-        except TypeError:
-            err('Container::toggle_zoom: %s is unable to handle zooming, for \
-            %s' % (self, widget))
-
     def zoom(self, widget, fontscale = False):
         """Zoom a terminal"""
         raise NotImplementedError('zoom')
@@ -154,33 +152,54 @@ class Container(object):
         """Unzoom a terminal"""
         raise NotImplementedError('unzoom')
 
-    def construct_confirm_close(self, window, reqtype):
+    def construct_confirm_close(self, window, child):
         """Create a confirmation dialog for closing things"""
-        
+        maker = Factory()
+
+        has_multiple_terms = False
+        if not maker.isinstance(child, 'Terminal'):
+            has_multiple_terms = True
+        elif maker.isinstance(self, 'Window'):
+            has_multiple_terms = self.is_zoomed()
+
         # skip this dialog if applicable
-        if self.config['suppress_multiple_term_dialog']:
+        if self.config['ask_before_closing'] == 'never':
             return Gtk.ResponseType.ACCEPT
-        
+        elif self.config['ask_before_closing'] == 'multiple_terminals':
+            if not has_multiple_terms:
+                return Gtk.ResponseType.ACCEPT
+
+        # text
+        confirm_button_text = (_('Close _Terminals') if has_multiple_terms
+                               else _('Close _Terminal'))
+        big_label_text = (_('Close multiple terminals?') if has_multiple_terms
+                          else _('Close terminal?'))
+
+        if not has_multiple_terms:
+            description_text = _('Do you really wish to close this terminal?')
+        elif maker.isinstance(self, 'Window'):
+            description_text = _('This window has several terminals open. Closing \
+the window will also close all terminals within it.')
+        elif maker.isinstance(self, 'Notebook'):
+            description_text = _('This tab has several terminals open. Closing \
+the tab will also close all terminals within it.')
+        else:
+            description_text = ''
+
+        # dialog GUI
         dialog = Gtk.Dialog(_('Close?'), window, Gtk.DialogFlags.MODAL)
         dialog.set_resizable(False)
     
         dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT)
         c_all = dialog.add_button(Gtk.STOCK_CLOSE, Gtk.ResponseType.ACCEPT)
         c_all.get_children()[0].get_children()[0].get_children()[1].set_label(
-                _('Close _Terminals'))
+                confirm_button_text)
     
-        primary = Gtk.Label(label=_('<big><b>Close multiple terminals?</b></big>'))
+        primary = Gtk.Label(label=_('<big><b>' + big_label_text + '</b></big>'))
         primary.set_use_markup(True)
         primary.set_alignment(0, 0.5)
-        if reqtype == 'window':
-            label_text = _('This window has several terminals open. Closing \
-the window will also close all terminals within it.')
-        elif reqtype == 'tab':
-            label_text = _('This tab has several terminals open. Closing \
-the tab will also close all terminals within it.')
-        else:
-            label_text = ''
-        secondary = Gtk.Label(label=label_text)
+
+        secondary = Gtk.Label(label=description_text)
         secondary.set_line_wrap(True)
                     
         labels = Gtk.VBox()
@@ -205,7 +224,8 @@ the tab will also close all terminals within it.')
         
         # set configuration
         self.config.base.reload()
-        self.config['suppress_multiple_term_dialog'] = checkbox.get_active()
+        if checkbox.get_active():
+            self.config['ask_before_closing'] = 'never'
         self.config.save()
 
         dialog.destroy()
@@ -246,13 +266,13 @@ the tab will also close all terminals within it.')
 
         return(terminals)
 
-    def describe_layout(self, count, parent, global_layout, child_order):
+    def describe_layout(self, count, parent, global_layout, child_order, save_cwd = False):
         """Describe our current layout"""
         layout = {}
         maker = Factory()
         mytype = maker.type(self)
         if not mytype:
-            err('unable to detemine own type. %s' % self)
+            err('unable to determine own type. %s' % self)
             return({})
 
         layout['type'] = mytype
@@ -283,7 +303,7 @@ the tab will also close all terminals within it.')
         if mytype == 'Notebook':
             labels = []
             last_active_term = []
-            for tabnum in xrange(0, self.get_n_pages()):
+            for tabnum in range(0, self.get_n_pages()):
                 page = self.get_nth_page(tabnum)
                 label = self.get_tab_label(page)
                 labels.append(label.get_custom_label())
@@ -309,7 +329,7 @@ the tab will also close all terminals within it.')
         child_order = 0
         for child in self.get_children():
             if hasattr(child, 'describe_layout'):
-                count = child.describe_layout(count, name, global_layout, child_order)
+                count = child.describe_layout(count, name, global_layout, child_order, save_cwd)
             child_order = child_order + 1
 
         return(count)
