@@ -18,6 +18,8 @@ from .terminator import Terminator
 from .plugin import PluginRegistry
 from .version import APP_NAME
 
+from .plugin import KeyBindUtil
+
 def get_color_string(widcol):
     return('#%02x%02x%02x' % (widcol.red>>8, widcol.green>>8, widcol.blue>>8))
 
@@ -233,14 +235,25 @@ class PrefsEditor:
         nb = guiget('notebook1')
         nb.set_current_page(cur_page)
 
+        self.config.base.save_config_with_suffix('_cur')
+
+    def on_destroy_event(self, _widget):
+        self.config.base.remove_config_with_suffix('_cur')
+
     def on_closebutton_clicked(self, _button):
         """Close the window"""
+        self.config.base.remove_config_with_suffix('_cur')
         terminator = Terminator()
         terminator.reconfigure()
         self.window.destroy()
         self.calling_window.preventHide = False
         del(self)
 
+    def on_restoreconfigbutton_clicked(self, _button):
+        """restore config to load time"""
+        self.config.base.restore_config_with_suffix('_cur')
+        self.on_closebutton_clicked(_button)
+
     def set_values(self):
         """Update the preferences window with all the configuration from
         Config()"""
@@ -295,6 +308,18 @@ class PrefsEditor:
             active = 0
         widget = guiget('winstatecombo')
         widget.set_active(active)
+        # Ask Before Closing
+        option = self.config['ask_before_closing']
+        if option == 'never':
+            active = 0
+        elif option == 'multiple_terminals':
+            active = 1
+        elif option == 'always':
+            active = 2
+        else:
+            active = 1 
+        widget = guiget('askbeforeclose')
+        widget.set_active(active)
         # Window borders
         widget = guiget('winbordercheck')
         widget.set_active(not self.config['borderless'])
@@ -337,6 +362,9 @@ class PrefsEditor:
         # DBus Server
         widget = guiget('dbuscheck')
         widget.set_active(self.config['dbus'])
+        # Detachable tabs
+        widget = guiget('detachable_tabs')
+        widget.set_active(self.config['detachable_tabs'])
         #Hide from taskbar
         widget = guiget('hidefromtaskbcheck')
         widget.set_active(self.config['hide_from_taskbar'])
@@ -353,7 +381,11 @@ class PrefsEditor:
         # title bar at bottom
         widget = guiget('title_at_bottom_checkbutton')
         widget.set_active(self.config['title_at_bottom'])
-        
+
+        # new tab after current tab
+        widget = guiget('new_tab_after_current_checkbutton')
+        widget.set_active(self.config['new_tab_after_current_tab'])
+
         #Always split with profile
         widget = guiget('always_split_with_profile')
         widget.set_active(self.config['always_split_with_profile'])
@@ -465,6 +497,15 @@ class PrefsEditor:
         liststore = widget.get_model()
         liststore.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         keybindings = self.config['keybindings']
+
+        keybindutil          = KeyBindUtil()
+        plugin_keyb_act      = keybindutil.get_all_act_to_keys()
+        plugin_keyb_desc     = keybindutil.get_all_act_to_desc()
+        #merge give preference to main bindings over plugin
+        keybindings          = {**plugin_keyb_act,  **keybindings}
+        self.keybindingnames = {**plugin_keyb_desc, **self.keybindingnames}
+        #dbg("appended actions %s names %s" % (keybindings, self.keybindingnames))
+
         for keybinding in keybindings:
             keyval = 0
             mask = 0
@@ -836,6 +877,10 @@ class PrefsEditor:
         self.config['dbus'] = widget.get_active()
         self.config.save()
 
+    def on_detachable_tabs_toggled(self, widget):
+        self.config['detachable_tabs'] = widget.get_active()
+        self.config.save()
+
     def on_disable_mousewheel_zoom_toggled(self, widget):
         """Ctrl+mousewheel zoom setting changed"""
         self.config['disable_mousewheel_zoom'] = widget.get_active()
@@ -881,6 +926,11 @@ class PrefsEditor:
         self.config['title_at_bottom'] = widget.get_active()
         self.config.save()
 
+    def on_new_tab_after_current_checkbutton_toggled(self, widget):
+        """New tab after current tab """
+        self.config['new_tab_after_current_tab'] = widget.get_active()
+        self.config.save()
+
     def on_always_split_with_profile_toggled(self, widget):
         """Always split with profile setting changed"""
         self.config['always_split_with_profile'] = widget.get_active()
@@ -958,11 +1008,6 @@ class PrefsEditor:
         self.config['login_shell'] = widget.get_active()
         self.config.save()
 
-    def on_scroll_background_checkbutton_toggled(self, widget):
-        """Scroll background setting changed"""
-        self.config['scroll_background'] = widget.get_active()
-        self.config.save()
-
     def on_scroll_on_keystroke_checkbutton_toggled(self, widget):
         """Scroll on keystrong setting changed"""
         self.config['scroll_on_keystroke'] = widget.get_active()
@@ -1381,8 +1426,6 @@ class PrefsEditor:
         value = int(value)          # Cast to int.
         if value > 20:
             value = 20
-        if value < 1:
-            value = 1
         self.config['handle_size'] = value
         self.config.save()
         guiget = self.builder.get_object
@@ -1466,7 +1509,20 @@ class PrefsEditor:
             value = 'normal'
         self.config['window_state'] = value
         self.config.save()
-
+    def on_askbeforeclose_changed(self, widget):
+        """Ask Before Close changed"""
+        selected = widget.get_active()
+        if selected == 0:
+            value = 'Never'
+        elif selected == 1:
+            value = 'Multiple Terminals'
+        elif selected == 2: 
+            value = 'Always'
+        else:
+            value = 'Multiple Terminals'
+        configval = value.lower().replace(" ","_")
+        self.config['ask_before_closing'] = configval
+        self.config.save()
     # helper function, not a signal
     def addprofile(self, name, toclone):
         """Add a profile"""
@@ -1563,6 +1619,14 @@ class PrefsEditor:
         (model, rowiter) = selected.get_selected()
         name = model.get_value(rowiter, 0)
 
+        config_layout  = self.config.base.get_layout(name)
+        dbg("layout from terminator:(%s)" % current_layout)
+        dbg("layout from config:(%s)" % config_layout)
+
+        self.config.copy_layout_item(config_layout, current_layout, 'directory')
+        self.config.copy_layout_item(config_layout, current_layout, 'command')
+        dbg("updated layout from terminator:(%s)" % current_layout)
+
         if self.config.replace_layout(name, current_layout):
             treeview.set_cursor(model.get_path(rowiter), column=treeview.get_column(0), start_editing=False)
         self.config.save()
@@ -1891,8 +1955,14 @@ class PrefsEditor:
         current_binding = liststore.get_value(liststore.get_iter(path), 0)
         parsed_accel = Gtk.accelerator_parse(accel)
 
+        keybindutil          = KeyBindUtil()
+        keybindings          = self.config["keybindings"]
+        #merge give preference to main bindings over plugin
+        plugin_keyb_act      = keybindutil.get_all_act_to_keys()
+        keybindings          = {**plugin_keyb_act,  **keybindings}
+
         duplicate_bindings = []
-        for conf_binding, conf_accel in self.config["keybindings"].items():
+        for conf_binding, conf_accel in keybindings.items():
             if conf_accel is None:
                 continue
 
@@ -1935,6 +2005,16 @@ class PrefsEditor:
         binding = liststore.get_value(liststore.get_iter(path), 0)
         accel = Gtk.accelerator_name(key, mods)
         self.config['keybindings'][binding] = accel
+
+        plugin_keyb_desc = keybindutil.get_act_to_desc(binding)
+        if plugin_keyb_desc:
+            dbg("modifying plugin binding: %s, %s, %s" %
+                                    (plugin_keyb_desc, binding, accel))
+            keybindutil.bindkey([plugin_keyb_desc, binding, accel])
+        else:
+            dbg("skipping: %s" % binding)
+            pass
+
         self.config.save()
 
     def on_cellrenderer_accel_cleared(self, liststore, path):
@@ -2116,7 +2196,7 @@ class LayoutEditor:
 
     def on_layout_profile_workingdir_activate(self, widget):
         """A new working directory has been entered for this item"""
-        workdir = widget.get_text()
+        workdir = os.path.expanduser(widget.get_text())
         layout = self.config.layout_get_config(self.layout_name)
         layout[self.layout_item]['directory'] = workdir
         self.config.save()
